#include "ParamFloat.h"
#include <algorithm>
#include <string>

namespace ffgl
{

ParamFloat::ParamFloat(const std::string& name, float defaultValue ) :
	ParamFloat( name, ParameterType::FF_TYPE_STANDARD, name, defaultValue )
{
}

ParamFloat::ParamFloat( const std::string& name, const std::string& uniformName, float defaultValue ) :
	ParamFloat( name, ParameterType::FF_TYPE_STANDARD, uniformName, defaultValue )
{
}

ParamFloat::ParamFloat( const std::string& name, ParameterType type, float defaultValue ) :
	ParamFloat( name, type, name, defaultValue )
{
}

ParamFloat::ParamFloat( const std::string& name, ParameterType type, const std::string& uniformName, float defaultValue ) :
	ParamBase( name, type, uniformName ),
	defaultValue( defaultValue )
{
	//TODO: Is this clamp necessary? Does it just get in the way?
	if( type == ParameterType::FF_TYPE_STANDARD )
	{
		// Clamp standard value to range [0, 1]
		if( defaultValue > 1.0f )
			ParamFloat::defaultValue = 1.0f;
		else if( defaultValue < 0.0f )
			ParamFloat::defaultValue = 0.0f;
	}

	Reset();
}

ParamFloat::~ParamFloat()
{
}

void ParamFloat::Reset()
{
	SetFloatValue( GetFloatDefault() );
}

const std::string& ParamFloat::GetDisplay() const
{
	static std::string displayString;
	displayString = std::to_string( GetFloatValue() );
	return displayString;
}

FFMixed ParamFloat::GetMixedDefault() const
{
	FFMixed retval;
	retval.FloatValue = GetFloatDefault();
	return retval;
}

FFMixed ParamFloat::GetMixedValue() const
{
	FFMixed retval;
	retval.FloatValue = GetFloatValue();
	return retval;
}

FFResult ParamFloat::SetMixedValue( const FFMixed mixed )
{
	SetFloatValue( mixed.FloatValue );
	return FF_SUCCESS;
}

float ParamFloat::GetFloatDefault() const
{
	return defaultValue;
}

float ParamFloat::GetFloatValue() const
{
	return value;
}

void ParamFloat::SetFloatValue( const float floatValue )
{
	value = floatValue;
}

}//End namespace ffgl